#include "../system/coordinates.h"
#include "../system/utilities.h"
#include "piece.h"
#include "pawn.h"
#include <vector>
#include <iostream>

/// @brief calculate a list of empty space coordinates needed for the move to be legal, assuming the move is possible
/// @param dest destination coordinates
/// @return a vector containing all coordinates to be verified
void pawn::needed_space(coordinates dest, std::vector<coordinates> *to_calculate){
    int diff_x = dest.get_x() - piece::coord.get_x();
    int diff_y = dest.get_y() - piece::coord.get_y();
    if(diff_x == 0){
        //Normal pawn move, destination case verified
        to_calculate->push_back(dest);
        // When moving two squares, also verify the intermediate one
        if(abs(diff_y) == 2){
            if(piece::white){
                to_calculate->push_back(coordinates(piece::coord.get_x(),piece::coord.get_y() + 1));
            }else{
                to_calculate->push_back(coordinates(piece::coord.get_x(),piece::coord.get_y() - 1));
            }
        }
    }else{
        //Capture pawn move, don't need any space
    }
}

/// @brief print the piece to the standard output
void pawn::print_piece(){
    if(piece::white){
        std::cout << "\u2659";
    }else{
        std::cout << "\u265F";
    }
}

/// @brief print the piece to the standard output in canonical form
void pawn::canonical_print_piece(){
    if(piece::white){
        std::cout << "wP";
    }else{
        std::cout << "bP";
    }
}

/// @brief called to move the pawn
/// @param dest the destination coordinates of the piece
/// @param debug_info print debug info to output when true
void pawn::move(coordinates dest, bool debug_info){
    if(!has_moved){
        legals.erase(legals.begin());
        has_moved = true;
    }
    coord.move(dest);
    if(debug_info){
        std::cout << "Piece move from : x=" << coord.get_x() << " y=" << coord.get_y() << "to : x=" << coord.get_x() << " y=" << coord.get_y() << std::endl;
        std::cout << "Piece info : white=" << white << std::endl;
        std::cout << "Legal moves :" << std::endl;
        for(int i = 0; i < legals.size(); i++){
            std::cout << "x=" << legals[i].get_x() << " y=" << legals[i].get_y() << std::endl;
        }
    }
}

/// @brief verify if the dest position is a valid attack move. The pawn need a special overrided function.
/// @param dest destination coordinates
/// @return if the move is within attack range
bool pawn::is_possible_attack(coordinates dest){
    if(white){
        return is_corresponding(coord,dest,coordinates(1,1)) || is_corresponding(coord,dest,coordinates(-1,1));
    }else{
        return is_corresponding(coord,dest,coordinates(1,-1)) || is_corresponding(coord,dest,coordinates(-1,-1));
    }
    
}

/// @brief retrieve the piece's legal moves
/// @return the legal moves vector
std::vector<coordinates> pawn::get_legals(){
    return possible_legals;
}