#ifndef PAWN
#define PAWN
#include "../system/coordinates.h"
#include "../system/utilities.h"
#include "piece.h"
#include <vector>

/// @brief the pawn piece class.
class pawn : public piece{
    private:
        // The standard moves and the attack moves all at once
        std::vector<coordinates> possible_legals;
    public:
        pawn(bool _white, coordinates _coord):
            piece(_white,_coord)
            {
                type = pawn_type;
                int direction = 0;
                if(piece::white){
                    direction = 1;
                }else{
                    direction = -1;
                }
                // The attack moves list is managed in the overrided is_possible_attack function.
                legals.push_back(coordinates(0,2*direction));
                possible_legals.push_back(coordinates(0,2*direction));
                legals.push_back(coordinates(0,direction));
                possible_legals.push_back(coordinates(0,direction));
                // Attack moves in possible_legals
                possible_legals.push_back(coordinates(1,direction));
                possible_legals.push_back(coordinates(-1,direction));
            }
        void needed_space(coordinates dest, std::vector<coordinates> *to_calculate) override;
        void print_piece() override;
        void canonical_print_piece() override;
        void move(coordinates dest, bool debug_info) override;
        bool is_possible_attack(coordinates dest) override;
        // We need to override because the pawn has attack moves that differs from legals (standard) move
        std::vector<coordinates> get_legals() override;
};

#endif