#include "piece.h"
#include "../system/utilities.h"
#include "../system/coordinates.h"
#include <iostream>

/// @brief the constructor for the piece class
/// @param _white indicates if the piece is white
piece::piece(bool _white, coordinates _coord):
            white(_white), coord(_coord)
            {
                legals = {};
                has_moved = false;
            }

/// @brief verify if a move is within piece range
/// @param dest destination coordinates
/// @return if the move is within range
bool piece::is_possible(coordinates dest){
    bool result = false;
    for(int i = 0; i < legals.size(); i++){
        result = result || (is_corresponding(coord,dest,legals[i]));
    }
    return result;
}

/// @brief verify if a move is within piece attack range
/// @param dest destination coordinates
/// @return if the move is within attack range
bool piece::is_possible_attack(coordinates dest){
    bool result = false;
    for(int i = 0; i < legals.size(); i++){
        result = result || (is_corresponding(coord,dest,legals[i]));
    }
    return result;
}

/// @brief calculate the space needed for the piece to move
/// @param dest destination point
/// @return a vector containing all spaces to be verified
void piece::needed_space(coordinates dest, std::vector<coordinates> *to_calculate){
    // By default, no space needed
}

/// @brief print the piece to the standard output
void piece::print_piece(){
    std::cout << "undefined piece";
}

/// @brief print the piece to the standard output in canonical form
void piece::canonical_print_piece(){
    std::cout << "undefined piece";
}

/// @brief called to move the piece
/// @param dest the destination coordinates of the piece
/// @param debug_info print debug info to output when true
void piece::move(coordinates dest,bool debug_info){
    //By default, only update has_moved
    has_moved = true;
    coord.move(dest);
    if(debug_info){
        std::cout << "Piece move from : x=" << coord.get_x() << " y=" << coord.get_y() << "to : x=" << coord.get_x() << " y=" << coord.get_y() << std::endl;
        std::cout << "Piece info : white=" << white << std::endl;
        std::cout << "Legal moves :" << std::endl;
        for(int i = 0; i < legals.size(); i++){
            std::cout << "x=" << legals[i].get_x() << " y=" << legals[i].get_y() << std::endl;
        }
    }
}

/// @brief retrieve the piece's coordinates
/// @return the coordinates of the piece
coordinates piece::get_coord(){
    return coord;
}

/// @brief retrieve the piece's color
/// @return the color of the piece
bool piece::get_color(){
    return white;
}

/// @brief retrieve if the piece has already moved during the game
/// @return the has_moved attribute
bool piece::get_has_moved(){
    return has_moved;
}

/// @brief retrieve the piece's type
/// @return the type of the piece
piece_type piece::get_type(){
    return type;
}

/// @brief retrieve the piece's legal moves
/// @return the legal moves vector
std::vector<coordinates> piece::get_legals(){
    return legals;
}