#ifndef PIECE
#define PIECE
#include "../system/coordinates.h"
#include "../system/utilities.h"
#include <vector>

/// @brief Represent a piece type. Undefined for default piece
enum piece_type {pawn_type, bishop_type, knight_type, rook_type, queen_type, king_type};

/// @brief a generic piece class. white is a boolean indicating the piece's color. legals is a vector indicating all legal movements. has_moved indicate if the piece has already move during this game
class piece{
    protected:
        bool white;
        coordinates coord;
        std::vector<coordinates> legals;
        bool has_moved;
        piece_type type;
    public:
        piece(bool _white, coordinates _coord);
        bool is_possible(coordinates dest);
        virtual bool is_possible_attack(coordinates dest);
        virtual void needed_space(coordinates dest, std::vector<coordinates> *to_calculate);
        virtual void print_piece();
        virtual void canonical_print_piece();
        virtual void move(coordinates dest,bool debug_info);
        coordinates get_coord();
        bool get_color();
        bool get_has_moved();
        piece_type get_type();
        virtual std::vector<coordinates> get_legals();
};

#endif