#include "../system/coordinates.h"
#include "../system/utilities.h"
#include "piece.h"
#include "queen.h"
#include <vector>
#include <iostream>

/// @brief calculate a list of empty space coordinates needed for the move to be legal, assuming the move is possible
/// @param dest destination coordinates
/// @return a vector containing all coordinates to be verified
void queen::needed_space(coordinates dest, std::vector<coordinates> *to_calculate){
    int diff_x = dest.get_x() - piece::coord.get_x();
    int diff_y = dest.get_y() - piece::coord.get_y();
    int coord_x = -1;
    int coord_y = -1;
    if(diff_x != 0 && diff_y != 0){
        //When the queen moves like a bishop
        for(int i = 1; i < abs(diff_x); i++){
            if(diff_x > 0){
                coord_x = piece::coord.get_x() + i;
            }else{
                coord_x = piece::coord.get_x() - i;
            }
            if(diff_y > 0){
                coord_y = piece::coord.get_y() + i;
            }else{
                coord_y = piece::coord.get_y() - i;
            }
            to_calculate->push_back(coordinates(coord_x,coord_y));
        }
    }else{
        //When the queen moves like a rook
        if(dest.get_x() > piece::coord.get_x()){
            //Moving right
            for(int i = piece::coord.get_x() + 1; i < dest.get_x(); i++){
                to_calculate->push_back(coordinates(i,piece::coord.get_y()));
            }
        }
        if(dest.get_x() < piece::coord.get_x()){
            //Moving left
            for(int i = piece::coord.get_x() - 1; i > dest.get_x(); i--){
                to_calculate->push_back(coordinates(i,piece::coord.get_y()));
            }
        }
        if(dest.get_y() > piece::coord.get_y()){
            //Moving right
            for(int i = piece::coord.get_y() + 1; i < dest.get_y(); i++){
                to_calculate->push_back(coordinates(piece::coord.get_x(),i));
            }
        }
        if(dest.get_y() < piece::coord.get_y()){
            //Moving left
            for(int i = piece::coord.get_y() - 1; i > dest.get_y(); i--){
                to_calculate->push_back(coordinates(piece::coord.get_x(),i));
            }
        }
    }
}

/// @brief print the piece to the standard output
void queen::print_piece(){
    if(piece::white){
        std::cout << "\u2655";
    }else{
        std::cout << "\u265B";
    }
}

/// @brief print the piece to the standard output in canonical form
void queen::canonical_print_piece(){
    if(piece::white){
        std::cout << "wQ";
    }else{
        std::cout << "bQ";
    }
}