#include "board.h"
#include <string>

board::board():
    last_move(0,0)
{
    without_attack_or_pawn_counter = 0;
    last_move_en_passant_eligible = false;
    white_pieces = {};
    for(int i = 0; i < 8; i++){
        white_pieces.push_back(new pawn(true,coordinates(i,1)));
    }
    white_pieces.push_back(new rook(true,coordinates(0,0)));
    white_pieces.push_back(new knight(true,coordinates(1,0)));
    white_pieces.push_back(new bishop(true,coordinates(2,0)));
    white_pieces.push_back(new queen(true,coordinates(3,0)));
    white_pieces.push_back(new king(true,coordinates(4,0)));
    white_pieces.push_back(new bishop(true,coordinates(5,0)));
    white_pieces.push_back(new knight(true,coordinates(6,0)));
    white_pieces.push_back(new rook(true,coordinates(7,0)));

    black_pieces = {};
    for(int i = 0; i < 8; i++){
        black_pieces.push_back(new pawn(false,coordinates(i,6)));
    }
    black_pieces.push_back(new rook(false,coordinates(0,7)));
    black_pieces.push_back(new knight(false,coordinates(1,7)));
    black_pieces.push_back(new bishop(false,coordinates(2,7)));
    black_pieces.push_back(new queen(false,coordinates(3,7)));
    black_pieces.push_back(new king(false,coordinates(4,7)));
    black_pieces.push_back(new bishop(false,coordinates(5,7)));
    black_pieces.push_back(new knight(false,coordinates(6,7)));
    black_pieces.push_back(new rook(false,coordinates(7,7)));

    // Initialize the board with empty pieces
    for(int i = 0; i < 8; i++){
        for(int j = 0; j < 8; j++){
            board_data[i][j] = nullptr;
        }
    }

    // Putting the pieces in place
    for(int i = 0; i < white_pieces.size(); i++){
        put(white_pieces[i]);
    }
    for(int i = 0; i < black_pieces.size(); i++){
        put(black_pieces[i]);
    }
}

/// @brief the copy constructor, which also copy the pieces pointers
/// @param other 
board::board(const board& other):
    last_move(other.last_move), last_move_en_passant_eligible(other.last_move_en_passant_eligible)
{
    without_attack_or_pawn_counter = 0;
    white_pieces = {};
    for(int i = 0; i < other.white_pieces.size(); i++){
        switch (other.white_pieces[i]->get_type())
        {
        case pawn_type:
            white_pieces.push_back(new pawn(other.white_pieces[i]->get_color(),other.white_pieces[i]->get_coord()));
            break;
        case bishop_type:
            white_pieces.push_back(new bishop(other.white_pieces[i]->get_color(),other.white_pieces[i]->get_coord()));
            break;
        case knight_type:
            white_pieces.push_back(new knight(other.white_pieces[i]->get_color(),other.white_pieces[i]->get_coord()));
            break;
        case rook_type:
            white_pieces.push_back(new rook(other.white_pieces[i]->get_color(),other.white_pieces[i]->get_coord()));
            break;
        case queen_type:
            white_pieces.push_back(new queen(other.white_pieces[i]->get_color(),other.white_pieces[i]->get_coord()));
            break;
        case king_type:
            white_pieces.push_back(new king(other.white_pieces[i]->get_color(),other.white_pieces[i]->get_coord()));
            break;
        }    
    }
    black_pieces = {};
    for(int i = 0; i < other.black_pieces.size(); i++){
        switch (other.black_pieces[i]->get_type())
        {
        case pawn_type:
            black_pieces.push_back(new pawn(other.black_pieces[i]->get_color(),other.black_pieces[i]->get_coord()));
            break;
        case bishop_type:
            black_pieces.push_back(new bishop(other.black_pieces[i]->get_color(),other.black_pieces[i]->get_coord()));
            break;
        case knight_type:
            black_pieces.push_back(new knight(other.black_pieces[i]->get_color(),other.black_pieces[i]->get_coord()));
            break;
        case rook_type:
            black_pieces.push_back(new rook(other.black_pieces[i]->get_color(),other.black_pieces[i]->get_coord()));
            break;
        case queen_type:
            black_pieces.push_back(new queen(other.black_pieces[i]->get_color(),other.black_pieces[i]->get_coord()));
            break;
        case king_type:
            black_pieces.push_back(new king(other.black_pieces[i]->get_color(),other.black_pieces[i]->get_coord()));
            break;
        }    
    }
    // Initialize the board with empty pieces
    for(int i = 0; i < 8; i++){
        for(int j = 0; j < 8; j++){
            board_data[i][j] = nullptr;
        }
    }

    // Putting the pieces in place
    for(int i = 0; i < white_pieces.size(); i++){
        put(white_pieces[i]);
    }
    for(int i = 0; i < black_pieces.size(); i++){
        put(black_pieces[i]);
    }
}

/// @brief put a piece onto the board according to its coordinates
/// @param to_put 
void board::put(piece* to_put){
    coordinates coord = to_put->get_coord();
    board_data[coord.get_x()][coord.get_y()] = to_put;
}

/// @brief print a neutral font color and background color new line
void print_neutral_endl(){
    terminal_reset_all();
    std::cout << std::endl;
}

/// @brief print a line separator with board matching background colors
/// @param inverted indicate if the line is inverted or not. Every even line is inverted to create the chess board
void print_line_separator(bool inverted){
    bool switcher = false;
    for(int i = 0; i < 8; i++){
        terminal_set_text_color(PIECES_COLOR);
        if((switcher&&!inverted)||(!switcher&&inverted)){
            terminal_set_background_color(BACKGROUND_COLOR1);
        }else{
            terminal_set_background_color(BACKGROUND_COLOR2);
        }
        std::cout << "     ";
        switcher = !switcher;
    }
}

/// @brief print the board
void board::print(){
    std::cout << "    a    b    c    d    e    f    g    h   ";
    print_neutral_endl();
    std::cout << "  ";
    terminal_set_background_color(BACKGROUND_COLOR1);
    terminal_set_text_color(PIECES_COLOR);
    bool line_switcher = true;
    bool column_switcher = false;
    print_line_separator(!line_switcher);
    print_neutral_endl();
    for(int y = 7; y > -1; y--){
        terminal_reset_all();
        std::cout << y + 1;
        std::cout << " ";
        if(line_switcher){
            terminal_set_background_color(BACKGROUND_COLOR1);
        }else{
            terminal_set_background_color(BACKGROUND_COLOR2);
        }
        column_switcher = true;
        terminal_set_text_color(PIECES_COLOR);
        if((line_switcher&&!column_switcher)||(!line_switcher&&column_switcher)){
                    terminal_set_background_color(BACKGROUND_COLOR1);
                }else{
                    terminal_set_background_color(BACKGROUND_COLOR2);
                }
        std::cout << "  ";
        for(int x = 0; x < 8; x++){
            if(board_data[x][y] == nullptr){
                std::cout << " ";
            }else{
                board_data[x][y]->print_piece();
            }
            if(x==7){
                std::cout << "  ";
                terminal_reset_all();
                std::cout << "  " << y + 1;
                print_neutral_endl();
            }else{
                std::cout << "  ";
                column_switcher = !column_switcher;
                if((line_switcher&&!column_switcher)||(!line_switcher&&column_switcher)){
                    terminal_set_background_color(BACKGROUND_COLOR1);
                }else{
                    terminal_set_background_color(BACKGROUND_COLOR2);
                }
                std::cout << "  ";
            }
        }
        if(y!=0){
            terminal_reset_all();
            std::cout << "  ";
            terminal_set_background_color(BACKGROUND_COLOR1);
            terminal_set_text_color(PIECES_COLOR);
            print_line_separator(line_switcher);
            print_neutral_endl();
        }
        
        line_switcher = !line_switcher;
    }
    std::cout << "    a    b    c    d    e    f    g    h   " << std::endl;
    // std::cout << "white:" << std::endl;
    // for(int i = 0; i < white_pieces.size(); i ++){
    //     white_pieces[i]->canonical_print_piece();
    //     std::cout << " x=" << white_pieces[i]->get_coord().get_x() << " y=" << white_pieces[i]->get_coord().get_y() << std::endl;
    // }
    // std::cout << "black:" << std::endl;
    // for(int i = 0; i < black_pieces.size(); i ++){
    //     black_pieces[i]->canonical_print_piece();
    //     std::cout << " x=" << black_pieces[i]->get_coord().get_x() << " y=" << black_pieces[i]->get_coord().get_y() << std::endl;
    // }
}

/// @brief print the board in canonical form
void board::canonical_print(){
    for(int y = 0; y < 8; y++){
        for(int x = 0; x < 8; x++){
            if(board_data[x][y]!=nullptr){
                board_data[x][y]->canonical_print_piece();
            }
            std::cout << ",";
        }
    }
}

/// @brief Erase a piece from its corresponding vector according to its coordinates
/// @param coord the coordinates of the piece to erase
void board::erase_piece(coordinates coord){
    // Reset the counter
    without_attack_or_pawn_counter = 0;
    bool piece_found = false;
    for(int i = 0; i < white_pieces.size() && !piece_found; i++){
        if(white_pieces[i]->get_coord().get_x() == coord.get_x() && white_pieces[i]->get_coord().get_y() == coord.get_y()){
            piece_found = true;
            white_pieces.erase(white_pieces.begin() + i);
        }
    }
    for(int i = 0; i < black_pieces.size() && !piece_found; i++){
        if(black_pieces[i]->get_coord().get_x() == coord.get_x() && black_pieces[i]->get_coord().get_y() == coord.get_y()){
            piece_found = true;
            black_pieces.erase(black_pieces.begin() + i);
        }
    }
}

/// @brief Move a piece from one position to another, supposing that the move is legal
/// @param move_from the initial coordinates of the piece
/// @param move_to the final coordinates of the piece
void board::move(coordinates move_from, coordinates move_to){
    // Increment the counter if not a pawn, else reset it
    if(board_data[move_from.get_x()][move_from.get_y()]->get_type() == pawn_type){
        without_attack_or_pawn_counter = 0;
    }else{
        without_attack_or_pawn_counter++;
    }
    if(board_data[move_to.get_x()][move_to.get_y()] != nullptr){
        //Attack case : remove the other piece
        erase_piece(move_to);
    }
    board_data[move_to.get_x()][move_to.get_y()] = board_data[move_from.get_x()][move_from.get_y()];
    board_data[move_to.get_x()][move_to.get_y()]->move(move_to,false);
    board_data[move_from.get_x()][move_from.get_y()] = nullptr;
    last_move.move(move_to);
    // last_move_en_passant_eligible attribute update if the piece moved is a pawn and it moves 2 cases.
    last_move_en_passant_eligible = (board_data[move_to.get_x()][move_to.get_y()]->get_type() == pawn_type)&&
    (abs(move_to.get_y() - move_from.get_y()) == 2);
}

bool board::is_en_passant_legal(coordinates move_from, coordinates move_to, bool white_move){
    // We successively check all conditions
    // First, checking the moving piece type
    if(board_data[move_from.get_x()][move_from.get_y()]->get_type() != pawn_type){
        return false;
    }
    // The en passant move is the same as the attack one
    if(!board_data[move_from.get_x()][move_from.get_y()]->is_possible_attack(move_to)){
        return false;
    }
    // The captured piece need to be the last played one, and be en_passant eligible
    if((last_move.get_x() != move_to.get_x()) || (last_move.get_y() != move_from.get_y()) || !last_move_en_passant_eligible){
        return false;
    }
    // The captured piece need to be one the same x of the destination, and on the same y of the starting point, and the opposite color
    if(board_data[move_to.get_x()][move_from.get_y()]->get_color() == white_move){
        return false;
    }
    // The captured piece need to be one the same x of the destination, and on the same y of the starting point, and a pawn
    if(board_data[move_to.get_x()][move_from.get_y()]->get_type() != pawn_type){
        return false;
    }
    return true;
}

/// @brief Check if a move is legal, without checking it in regards to king check state
/// @param move_from the initial position
/// @param move_to the final position
/// @param white_move indicate if the current active player is White
/// @return a boolean indicating if the move is legal
bool board::is_legal(coordinates move_from, coordinates move_to, bool white_move){
    //Check the move isn't off the board
    if(move_from.get_x() < 0 || move_from.get_x() > 7 || move_from.get_y() < 0 || move_from.get_y() > 7 ||
    move_to.get_x() < 0 || move_to.get_x() > 7 || move_to.get_y() < 0 || move_to.get_y() > 7){
        return false;
    }

    if(board_data[move_from.get_x()][move_from.get_y()] == nullptr){
        //Moving from an empty space, wrong move
        return false;
    }

    if(board_data[move_from.get_x()][move_from.get_y()]->get_color() != white_move){
        //Moving a piece that the player isn't owning, wrong move
        return false;
    }
    std::vector<coordinates> empty_check = {};
    if(board_data[move_to.get_x()][move_to.get_y()] == nullptr){
        //Destination case is empty
        if(is_en_passant_legal(move_from,move_to,white_move)){
            // The en-passant special case, we capture the corresponding pawn
            erase_piece(coordinates(move_to.get_x(),move_from.get_y()));
            board_data[move_to.get_x()][move_from.get_y()] = nullptr;
        }else{
            if(!board_data[move_from.get_x()][move_from.get_y()]->is_possible(move_to)){
                // The normal move isn't possible
                return false;
            }
        }
        
        board_data[move_from.get_x()][move_from.get_y()]->needed_space(move_to, &empty_check);


    }else{
        if(board_data[move_to.get_x()][move_to.get_y()]->get_color() == white_move){
            //Destination case support a piece of the same color, wrong move
            return false;
        }else{
            //Destination case support a piece of a different color
            if(!board_data[move_from.get_x()][move_from.get_y()]->is_possible_attack(move_to)){
                // The attack move isn't possible
                return false;
            }
            board_data[move_from.get_x()][move_from.get_y()]->needed_space(move_to, &empty_check);
        }
    }
    bool result = true;
    for(int i = 0; i < empty_check.size(); i++){
        result = result && (board_data[empty_check[i].get_x()][empty_check[i].get_y()] == nullptr);
    }
    return result;
    
}

/// @brief Retrieve coordinates from a string
/// @param move a string with the pattern "a6" (half a input)
/// @return coordinates corresponding to the string
coordinates get_coord_from_string(std::string move){
    int result_y = atoi(&move[1]) - 1;
    int result_x = 0;
    std::vector<char> x_traduction = {'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h'};
    for(int i = 0; i < 8; i++){
        if(move[0] == x_traduction[i]){
            result_x = i;
        }
    }
    return coordinates(result_x,result_y);
}

/// @brief retrieve the king coordinates
/// @param white_king if the selected king is white
/// @return the coordinates of the selected king
coordinates board::get_king_coord(bool white_king){
    coordinates result (0,0);
    if(white_king){
        for(int i = 0; i < white_pieces.size(); i++){
            if(white_pieces[i]->get_type() == king_type){
                result = white_pieces[i]->get_coord();
            }
        }
    }else{
        for(int i = 0; i < black_pieces.size(); i++){
            if(black_pieces[i]->get_type() == king_type){
                result = black_pieces[i]->get_coord();
            }
        }
    }
    return result;
}

/// @brief check if the selected player is in check state
/// @param white_king indicate if the king checked is white
/// @return a boolean indicating if a check is present
bool board::is_check(bool white_king){
    bool result = false;
    if(white_king){
        for(int i = 0; i < black_pieces.size(); i++){
            if(black_pieces[i]->get_type() == queen_type){
            }
            result = result || is_legal(black_pieces[i]->get_coord(),get_king_coord(white_king),!white_king);
        }
    }else{
        for(int i = 0; i < white_pieces.size(); i++){
            result = result || is_legal(white_pieces[i]->get_coord(),get_king_coord(white_king),!white_king);
        }
    }
    return result;
}

/// @brief check if the king-side castling move is legal, without checking it in regards to king check state
/// @param white_move if the current player is White
/// @return a boolean equals to true if the king-side castling move is legal
bool board::is_king_castling_legal(bool white_move){
    int y;
    if(white_move){
        y = 0;
    }else{
        y = 7;
    }
    //We first check that the two pieces are in place
    if(board_data[4][y] == nullptr || board_data[7][y] == nullptr){
        return false;
    }
    //We then check that the two pieces didn't moved. Don't need to check the type since we check they didn't moved
    if(board_data[4][y]->get_has_moved() || board_data[7][y]->get_has_moved()){
        return false;
    }
    //We then check if there is space to perform the castling
    return (board_data[5][y] == nullptr) && (board_data[6][y] == nullptr);
}

/// @brief check if the queen-side castling move is legal, without checking it in regards to king check state
/// @param white_move if the current player is White
/// @return a boolean equals to true if the queen-side castling move is legal
bool board::is_queen_castling_legal(bool white_move){
    int y;
    if(white_move){
        y = 0;
    }else{
        y = 7;
    }
    //We first check that the two pieces are in place
    if(board_data[4][y] == nullptr || board_data[0][y] == nullptr){
        return false;
    }
    //We then check that the two pieces didn't moved
    if(board_data[4][y]->get_has_moved() || board_data[0][y]->get_has_moved()){
        return false;
    }
    //We then check if there is space to perform the castling
    return (board_data[1][y] == nullptr) && (board_data[2][y] == nullptr) && (board_data[3][y] == nullptr);
}

/// @brief perform the king-side castling move
/// @param white_move true if the player is White
void board::king_castle(bool white_move){
    int y;
    if(white_move){
        y = 0;
    }else{
        y = 7;
    }
    move(coordinates(4,y),coordinates(6,y));
    move(coordinates(7,y),coordinates(5,y));
}

/// @brief perform the queen-side castling move
/// @param white_move true if the player is White
void board::queen_castle(bool white_move){
    int y;
    if(white_move){
        y = 0;
    }else{
        y = 7;
    }
    move(coordinates(4,y),coordinates(2,y));
    move(coordinates(0,y),coordinates(3,y));
}



/// @brief Perform a promotion on the specified coordinates. Don't check anything
/// @param to_promote the coordinates of the piece to promote
/// @param new_type the new type of the piece
void board::promote(coordinates to_promote, piece_type new_type){
    bool color = board_data[to_promote.get_x()][to_promote.get_y()]->get_color();
    erase_piece(to_promote);
    if(color){
        switch (new_type)
        {
        case rook_type:
            white_pieces.push_back(new rook(color,to_promote));
            break;
        case bishop_type:
            white_pieces.push_back(new bishop(color,to_promote));
            break;
        case knight_type:
            white_pieces.push_back(new knight(color,to_promote));
            break;
        case queen_type:
            white_pieces.push_back(new queen(color,to_promote));
            break;
        }
        //Put the newly added piece on the board
        put(white_pieces[white_pieces.size() - 1]);
    }else{
        switch (new_type)
        {
        case rook_type:
            black_pieces.push_back(new rook(color,to_promote));
            break;
        case bishop_type:
            black_pieces.push_back(new bishop(color,to_promote));
            break;
        case knight_type:
            black_pieces.push_back(new knight(color,to_promote));
            break;
        case queen_type:
            black_pieces.push_back(new queen(color,to_promote));
            break;
        }
        //Put the newly added piece on the board
        put(black_pieces[black_pieces.size() - 1]);
    }
    
}

/// @brief check if a promotion is needed
/// @param coord the coordinates to check
/// @return if a promotion is needed at the selected coordinates
bool board::is_promotion_needed(coordinates coord){
    return((board_data[coord.get_x()][coord.get_y()]->get_type() == pawn_type) && ((coord.get_y() == 0)||(coord.get_y() == 7)));
}

/// @brief check if the other player can move without being in a check state
/// @param white_move true if the player that just played is White
/// @return true if the opposite player can't move without being in check state
bool board::is_checkmate_or_pat(bool white_move){
    // 50 moves rule
    if(without_attack_or_pawn_counter >= 50){
        return true;
    }
    bool result = true;
    // The copy of the copy
    board *chess_board_copy;
    // A check board copy
    board *chess_board;
    coordinates start(0,0);
    coordinates end(0,0);
    if(white_move){
        for(int i = 0; i < black_pieces.size(); i++){
            // For each piece, we test all moves
            // We don't have to check the castling moves because they're unauthorized when in a check state
            for(int j = 0; j < black_pieces[i]->get_legals().size(); j++){
                // For a move, we check if it's still in check
                chess_board = new board(*this);
                start.move(black_pieces[i]->get_coord());
                // Simulate a movement
                end.move(coordinates(black_pieces[i]->get_coord().get_x() + black_pieces[i]->get_legals()[j].get_x(),
                black_pieces[i]->get_coord().get_y() + black_pieces[i]->get_legals()[j].get_y()));
                if(chess_board->is_legal(start,end,!white_move)){
                    //Apply the movement on a chess board copy and verify the check state of the current player's king
                    chess_board_copy = new board(*chess_board);
                    chess_board_copy->move(start,end);
                    if(chess_board_copy->is_check(!white_move)){
                        //The move is making the active player in check state, illegal
                    }else{
                        //The move is legal and the player isn't in check, therefore it isn't a checkmate.
                        result = false;
                    }
                    delete chess_board_copy;
                }
                delete chess_board;
            }
        }
    }else{
        for(int i = 0; i < white_pieces.size(); i++){
            // For each piece, we test all moves
            // We don't have to check the castling moves because they're unauthorized when in a check state
            for(int j = 0; j < white_pieces[i]->get_legals().size(); j++){
                // For a move, we check if it's still in check
                chess_board = new board(*this);
                start.move(white_pieces[i]->get_coord());
                // Simulate a movement
                end.move(coordinates(white_pieces[i]->get_coord().get_x() + white_pieces[i]->get_legals()[j].get_x(),
                white_pieces[i]->get_coord().get_y() + white_pieces[i]->get_legals()[j].get_y()));
                if(chess_board->is_legal(start,end,!white_move)){
                    //Apply the movement on a chess board copy and verify the check state of the current player's king
                    chess_board_copy = new board(*chess_board);
                    chess_board_copy->move(start,end);
                    if(chess_board_copy->is_check(!white_move)){
                        //The move is making the active player in check state, illegal
                    }else{
                        //The move is legal and the player isn't in check, therefore it isn't a checkmate.
                        result = false;
                    }
                    delete chess_board_copy;
                }
            delete chess_board;
            }
        }
    }
    return result;
}