#ifndef BOARD
#define BOARD
#include <iostream>
#include <vector>
#include "utilities.h"
#include "coordinates.h"
#include "../pieces/piece.h"
#include "../pieces/pawn.h"
#include "../pieces/bishop.h"
#include "../pieces/knight.h"
#include "../pieces/rook.h"
#include "../pieces/queen.h"
#include "../pieces/king.h"
#include <regex>
#include <string>
#include "config.h"

/// @brief A class representing the chess board
class board{
    private:
        piece * board_data[8][8];
        std::vector<piece*> white_pieces;
        std::vector<piece*> black_pieces;
        coordinates last_move;
        bool last_move_en_passant_eligible;
        int without_attack_or_pawn_counter;
    public:
        board();
        board(const board& other);
        void print();
        void canonical_print();
        void put(piece* to_put);
        void move(coordinates move_from, coordinates move_to);
        bool is_legal(coordinates move_from, coordinates move_to, bool white_move);
        void erase_piece(coordinates coord);
        coordinates get_king_coord(bool white_king);
        bool is_check(bool white_king);
        bool is_en_passant_legal(coordinates move_from, coordinates move_to, bool white_move);
        bool is_king_castling_legal(bool white_move);
        bool is_queen_castling_legal(bool white_move);
        void king_castle(bool white_move);
        void queen_castle(bool white_move);
        void promote(coordinates to_promote, piece_type new_type);
        bool is_promotion_needed(coordinates coord);
        bool is_checkmate_or_pat(bool white_move);
};

void print_neutral_endl();
void print_line_separator(bool inverted);
coordinates get_coord_from_string(std::string move);

#endif