#include "board.h"
#include <iostream>
#include <string>

/// @brief an enumeration to represent the game result
enum winner_type {win_white, win_black, draw, interrupt};

/// @brief The main function
/// @return nothing
int main(){
    board chess_board;
    std::string movement;
    bool game_end = false;
    bool correct_move = false;
    coordinates start(0,0);
    coordinates end(0,0);
    bool white_move = true;
    winner_type game_result;
    board *chess_board_copy;
    while(!game_end){
        chess_board.print();
        movement = "";
        correct_move = false;
        while(!correct_move){
            if(white_move){
                terminal_set_background_color(BACKGROUND_COLOR1);
                terminal_set_text_color(PIECES_COLOR);
                std::cout << "White";
                terminal_reset_all();
                std::cout << " turn. ";
            }else{
                terminal_set_background_color(PIECES_COLOR);
                terminal_set_text_color(BACKGROUND_COLOR1);
                std::cout << "Black";
                terminal_reset_all();
                std::cout << " turn. ";
            }
            // Check warning for current player king
            if(chess_board.is_check(white_move)){
                std::cout << std::endl;
                terminal_set_text_color(WARNING_TEXT_COLOR);
                std::cout << "Warning : Check state" << std::endl;
                terminal_reset_all();
            }
            std::cout << "What move ? " << std::endl;
            std::cin >> movement;
            if(movement == "/quit" || movement == "/resign" || movement == "/draw"){
                //The movement input is correct and a game ending one
                game_end = true;
                correct_move = true;
                if(movement == "/quit"){
                    game_result = interrupt;
                }
                if(movement == "/resign"){
                    if(white_move){
                        game_result = win_black;
                    }else{
                        game_result = win_white;
                    }
                }
                if(movement == "/draw"){
                    game_result = draw;
                }
            }else{
                if(correct_input(movement)){
                    //The movement input is correct
                    if(standard_input(movement)){
                        //Standard movement case
                        coordinates start = get_coord_from_string(movement.substr(0, 2));
                        coordinates end = get_coord_from_string(movement.substr(2, 2));
                        if(chess_board.is_legal(start,end,white_move)){
                            //Apply the movement on a chess board copy and verify the check state of the current player's king
                            chess_board_copy = new board(chess_board);
                            chess_board_copy->move(start,end);
                            if(chess_board_copy->is_check(white_move)){
                                //The move is making the active player in check state, illegal
                                std::cout << "Illegal move. Your king will be checked" << std::endl;
                            }else{
                                chess_board.move(start,end);
                                correct_move = true;
                                if(chess_board.is_promotion_needed(end)){
                                    chess_board.print();
                                    // Promotion selection
                                    std::cout << "Promotion. Choose a piece to substitue the pawn : B/N/R/Q" << std::endl;
                                    std::string chose_promotion = "";
                                    while((chose_promotion != "B") && (chose_promotion != "N")
                                    && (chose_promotion != "R") && (chose_promotion != "Q")){
                                        std::cin >> chose_promotion;
                                        if((chose_promotion != "B") && (chose_promotion != "N")
                                        && (chose_promotion != "R") && (chose_promotion != "Q")){
                                            std::cout << "Wrong input format. Input to be among : B,N,R and Q" << std::endl;
                                        }
                                    }
                                    // We promote to the right piece
                                    if(chose_promotion == "B"){
                                        chess_board.promote(end,bishop_type);
                                    }
                                    if(chose_promotion == "N"){
                                        chess_board.promote(end,knight_type);
                                    }
                                    if(chose_promotion == "R"){
                                        chess_board.promote(end,rook_type);
                                    }
                                    if(chose_promotion == "Q"){
                                        chess_board.promote(end,queen_type);
                                    }
                                }
                            }
                            delete chess_board_copy;
                        }else{
                            std::cout << "Illegal move." << std::endl;
                        }
                    }
                    if(king_castle_input(movement)){
                        //King-side castling move
                        if(chess_board.is_king_castling_legal(white_move)){
                            //Apply the movement on a chess board copy and verify the check state of the current player's king
                            chess_board_copy = new board(chess_board);
                            chess_board_copy->king_castle(white_move);
                            if(chess_board_copy->is_check(white_move)){
                                //The move is making the active player in check state, illegal
                                std::cout << "Illegal move. Your king will be checked" << std::endl;
                            }else{
                                chess_board.king_castle(white_move);
                                correct_move = true;
                            }
                            delete chess_board_copy;
                        }else{
                            std::cout << "Illegal move." << std::endl;
                        }
                    }
                    if(queen_castle_input(movement)){
                        //Queen-side castling move
                        if(chess_board.is_queen_castling_legal(white_move)){
                            //Apply the movement on a chess board copy and verify the check state of the current player's king
                            chess_board_copy = new board(chess_board);
                            chess_board_copy->queen_castle(white_move);
                            if(chess_board_copy->is_check(white_move)){
                                //The move is making the active player in check state, illegal
                                std::cout << "Illegal move. Your king will be checked" << std::endl;
                            }else{
                                chess_board.queen_castle(white_move);
                                correct_move = true;
                            }
                            delete chess_board_copy;
                        }else{
                            std::cout << "Illegal move." << std::endl;
                        }
                    }
                }else{
                    //The movement input is wrong
                    std::cout << "Wrong input format. Format need to be in algrebic notation : b1c3 for example. 0-0-0 and 0-0 for the castlings" << std::endl;
                }
            }
        }
        //Checkmate and pat detection
        if(chess_board.is_checkmate_or_pat(white_move)){
            game_end = true;
            chess_board.print();
            if(white_move){
                if(chess_board.is_check(!white_move)){
                    game_result = win_white;
                    terminal_set_background_color(BACKGROUND_COLOR1);
                    terminal_set_text_color(PIECES_COLOR);
                    std::cout << "White win.";
                    print_neutral_endl();
                }else{
                    game_result = draw;
                    std::cout << "Draw.";
                    print_neutral_endl();
                }
            }else{
                if(chess_board.is_check(!white_move)){
                    game_result = win_black;
                    terminal_set_background_color(PIECES_COLOR);
                    terminal_set_text_color(BACKGROUND_COLOR1);
                    std::cout << "Black win.";
                    print_neutral_endl();
                }else{
                    game_result = draw;
                    std::cout << "Draw.";
                    print_neutral_endl();
                }
            }
        }
        white_move = !white_move;
    }
    chess_board.canonical_print();
    switch (game_result)
    {
    case win_white:
        std::cout << " 1-0" << std::endl;
        break;

    case win_black:
        std::cout << " 0-1" << std::endl;
        break;

    case draw:
        std::cout << " 1/2-1/2" << std::endl;
        break;

    case interrupt:
        std::cout << " ?-?" << std::endl;
        break;

    }
}