#include "utilities.h"
#include "coordinates.h"
#include <iostream>

/// @brief verify if a movement is corresponding to a starting and ending coordinates
/// @param start starting coordinates
/// @param end ending coordinates
/// @param movement the move to be verified, represente as coordinates
/// @return a boolean value corresponding to the move being valid or not
bool is_corresponding(coordinates start, coordinates end, coordinates movement){
    return (end.get_x() - start.get_x() == movement.get_x()) && (end.get_y() - start.get_y() == movement.get_y());
}

/// @brief set the terminal font color to the input
/// @param color 
void terminal_set_text_color(terminal_color color){
    switch (color)
    {
        case terminal_black:
            std::cout << "\033[30m";
            break;
        case terminal_red:
            std::cout << "\033[31m";
            break;
        case terminal_green:
            std::cout << "\033[32m";
            break;
        case terminal_yellow:
            std::cout << "\033[33m";
            break;
        case terminal_blue:
            std::cout << "\033[34m";
            break;
        case terminal_magenta:
            std::cout << "\033[35m";
            break;
        case terminal_cyan:
            std::cout << "\033[36m";
            break;
        case terminal_white:
            std::cout << "\033[37m";
            break;
    }
}

/// @brief set the terminal background color to the input
/// @param color 
void terminal_set_background_color(terminal_color color){
    switch (color)
    {
        case terminal_black:
            std::cout << "\033[40m";
            break;
        case terminal_red:
            std::cout << "\033[41m";
            break;
        case terminal_green:
            std::cout << "\033[42m";
            break;
        case terminal_yellow:
            std::cout << "\033[43m";
            break;
        case terminal_blue:
            std::cout << "\033[44m";
            break;
        case terminal_magenta:
            std::cout << "\033[45m";
            break;
        case terminal_cyan:
            std::cout << "\033[46m";
            break;
        case terminal_white:
            std::cout << "\033[47m";
            break;
    }
}

/// @brief reset all terminal attributes
void terminal_reset_all(){
    std::cout << "\033[0m";
}

/// @brief Check if an input is conform
/// @param cmd the input string
/// @return a boolean indicating if the input is conform
bool correct_input(std::string const & cmd){
    std::regex mouvmtpattern("[a-h][1-8][a-h][1-8]");
    std::regex kingcastlemouvmtpattern("(O|o|0)-(O|o|0)");
    std::regex queencastlemouvmtpattern("(O|o|0)-(O|o|0)-(O|o|0)");
    return regex_match(cmd,mouvmtpattern) || regex_match(cmd,kingcastlemouvmtpattern) || regex_match(cmd,queencastlemouvmtpattern);
}

/// @brief Check if an input corresponds to a standard move
/// @param cmd the input string
/// @return a boolean indicating if the input is a standard move
bool standard_input(std::string const & cmd){
    std::regex mouvmtpattern("[a-h][1-8][a-h][1-8]");
    return regex_match(cmd,mouvmtpattern);
}

/// @brief Check if an input correponds to a king castling move
/// @param cmd the input string
/// @return a boolean indicating if the input is a king castling move
bool king_castle_input(std::string const & cmd){
    std::regex kingcastlemouvmtpattern("(O|o|0)-(O|o|0)");
    return regex_match(cmd,kingcastlemouvmtpattern);
}

/// @brief Check if an input corresponds to a queen castling move
/// @param cmd the input string
/// @return a boolean indicating if the input is a queen castling move
bool queen_castle_input(std::string const & cmd){
    std::regex queencastlemouvmtpattern("(O|o|0)-(O|o|0)-(O|o|0)");
    return regex_match(cmd,queencastlemouvmtpattern);
}

/// @brief Print a message to the standard output : "test i". Used for debug purposes
/// @param i the number that appear on the debug message
void print_debug(int i){
    std::cout << "test" << i << std::endl;
}